@@ -20,11 +20,13 @@
 import contextlib
 import logging
 import math
+import urllib
 
+from eventlet import tpool
 from oslo_config import cfg
 from oslo_utils import encodeutils
+from oslo_utils import eventletutils
 from oslo_utils import units
-from six.moves import urllib
 
 from glance_store import capabilities
 from glance_store.common import utils
@@ -133,6 +135,13 @@ Related options:
 
 """),
     cfg.IntOpt('rados_connect_timeout', default=0,
+               deprecated_for_removal=True,
+               deprecated_since='Zed',
+               deprecated_reason="""
+This option has not had any effect in years. Users willing to set a timeout for
+connecting to the Ceph cluster should use 'client_mount_timeout' in Ceph's
+configuration file.
+""",
                help="""
 Timeout value for connecting to Ceph cluster.
 
@@ -283,12 +292,18 @@ class Store(driver.Store):
     def get_schemes(self):
         return ('rbd',)
 
+    def RBDProxy(self):
+        if eventletutils.is_monkey_patched('thread'):
+            return tpool.Proxy(rbd.RBD())
+        else:
+            return rbd.RBD()
+
     @contextlib.contextmanager
     def get_connection(self, conffile, rados_id):
         client = rados.Rados(conffile=conffile, rados_id=rados_id)
 
         try:
-            client.connect(timeout=self.connect_timeout)
+            client.connect()
         except (rados.Error, rados.ObjectNotFound) as e:
             if self.backend_group and len(self.conf.enabled_backends) > 1:
                 reason = _("Error in store configuration: %s") % e
@@ -311,6 +326,13 @@ class Store(driver.Store):
         this method. If the store was not able to successfully configure
         itself, it should raise `exceptions.BadStoreConfiguration`
         """
+        if rbd is None or rados is None:
+            reason = _("The required libraries(rbd and rados) are not "
+                       "available")
+            LOG.error(reason)
+            raise exceptions.BadStoreConfiguration(store_name='rbd',
+                                                   reason=reason)
+
         try:
             if self.backend_group:
                 chunk = getattr(self.conf,
@@ -319,8 +341,6 @@ class Store(driver.Store):
                 user = getattr(self.conf, self.backend_group).rbd_store_user
                 conf_file = getattr(self.conf,
                                     self.backend_group).rbd_store_ceph_conf
-                connect_timeout = getattr(
-                    self.conf, self.backend_group).rados_connect_timeout
                 thin_provisioning = getattr(self.conf,
                                             self.backend_group).\
                     rbd_thin_provisioning
@@ -329,7 +349,6 @@ class Store(driver.Store):
                 pool = self.conf.glance_store.rbd_store_pool
                 user = self.conf.glance_store.rbd_store_user
                 conf_file = self.conf.glance_store.rbd_store_ceph_conf
-                connect_timeout = self.conf.glance_store.rados_connect_timeout
                 thin_provisioning = \
                     self.conf.glance_store.rbd_thin_provisioning
 
@@ -343,7 +362,6 @@ class Store(driver.Store):
             self.pool = str(pool)
             self.user = str(user)
             self.conf_file = str(conf_file)
-            self.connect_timeout = connect_timeout
         except cfg.ConfigFileValueError as e:
             reason = _("Error in store configuration: %s") % e
             LOG.error(reason)
@@ -421,12 +439,12 @@ class Store(driver.Store):
 
         :returns: `glance_store.rbd.StoreLocation` object
         """
-        librbd = rbd.RBD()
         features = conn.conf_get('rbd_default_features')
         if ((features is None) or (int(features) == 0)):
             features = rbd.RBD_FEATURE_LAYERING
-        librbd.create(ioctx, image_name, size, order, old_format=False,
-                      features=int(features))
+        self.RBDProxy().create(ioctx, image_name, size, order,
+                               old_format=False,
+                               features=int(features))
         return StoreLocation({
             'fsid': fsid,
             'pool': self.pool,
@@ -461,14 +479,6 @@ class Store(driver.Store):
                     if snapshot_name is not None:
                         with rbd.Image(ioctx, image_name) as image:
                             try:
-                                # NOTE(abhishekk): Check whether snapshot
-                                # has any external references
-                                if self._snapshot_has_external_reference(
-                                        image, snapshot_name):
-                                    raise rbd.ImageBusy(
-                                        "Image snapshot has external "
-                                        "references.")
-
                                 self._unprotect_snapshot(image, snapshot_name)
                                 image.remove_snap(snapshot_name)
                             except rbd.ImageNotFound as exc:
@@ -486,12 +496,21 @@ class Store(driver.Store):
                                 raise exceptions.InUseByStore()
 
                     # Then delete image.
-                    rbd.RBD().remove(ioctx, image_name)
+                    self.RBDProxy().remove(ioctx, image_name)
                 except rbd.ImageHasSnapshots:
-                    log_msg = (_LW("Remove image %(img_name)s failed. "
-                                   "It has snapshot(s) left.") %
+                    log_msg = (_LW("Unable to remove image %(img_name)s: it "
+                                   "has snapshot(s) left; trashing instead") %
                                {'img_name': image_name})
                     LOG.warning(log_msg)
+                    with rbd.Image(ioctx, image_name) as image:
+                        try:
+                            rbd.RBD().trash_move(ioctx, image_name)
+                            LOG.debug('Moved %s to trash', image_name)
+                        except rbd.ImageBusy:
+                            LOG.warning(_('Unable to move in-use image to '
+                                          'trash'))
+                            raise exceptions.InUseByStore()
+                        return
                     raise exceptions.HasSnapshot()
                 except rbd.ImageBusy:
                     log_msg = (_LW("Remove image %(img_name)s failed. "
@@ -651,7 +670,7 @@ class Store(driver.Store):
         # Add store backend information to location metadata
         metadata = {}
         if self.backend_group:
-            metadata['store'] = u"%s" % self.backend_group
+            metadata['store'] = self.backend_group
 
         return (loc.get_uri(),
                 image_size,
